import os
import subprocess

from amaranth.build import *
from amaranth.vendor import LatticeMachXO2Platform
from .resources import *


__all__ = ["MachXO2_7000HE_BreakoutPlatform", "MachXO2_1200ZE_BreakoutPlatform"]


# https://www.latticesemi.com/Products/DevelopmentBoardsAndKits/MachXO2BreakoutBoard
class MachXO2_7000HE_BreakoutPlatform(LatticeMachXO2Platform):
    device         = "LCMXO2-7000HE"
    package        = "TG144"
    default_clk    = "OSCH"
    osch_frequency = 2.08 # documented default; see amaranth.vendor.lattice_machxo_2_3l for more options
    speed          = "4"
    resources      = [
        *LEDResources(pins="97 98 99 100 104 105 106 107", invert=True),
    ]

    # Connectable to the FTDI UART but disconnected by default. Populate R14-R21 to connect.
    # Use `p.add_resource(p.serial)` to add the resource.
    serial = UARTResource(0, rx="73", tx="74", rts="75", cts="76", dtr="81", dsr="77", dcd="78", role="dte")

    connectors     = [
        Connector("j", 2, # J2
                  "- - "
                  "109 - " # 110: INITn
                  "111 112 "
                  "- - "
                  "113 114 "
                  "115 117 "
                  "119 120 "
                  "- - "
                  "121 122 "
                  "125 126 "
                  "127 128 "
                  "- - "
                  "- - " # 130: TMS, 131: TCK
                  "132 133 "
                  "- - " # 136: TDI, 137: TDO
                  "- - "
                  "138 139 "
                  "140 141 "
                  "142 143 "
                  "- - "),
        Connector("j", 3, # J3
                  "- - "
                  "- - "
                  "74 73 "
                  "76 75 "
                  "- - "
                  "78 77 "
                  "82 81 "
                  "- - "
                  "84 83 "
                  "86 85 "
                  "- - "
                  "92 91 "
                  "94 93 "
                  "- - "
                  "96 95 "
                  "98 97 "
                  "- - "
                  "100 99 "
                  "105 104 "
                  "107 106 "),
        Connector("j", 4, # J4
                  "- - "
                  "- - "
                  "1 2 "
                  "3 4 "
                  "5 6 "
                  "9 10 "
                  "- - "
                  "11 12 "
                  "13 14 "
                  "- - "
                  "19 20 "
                  "21 22 "
                  "- - "
                  "23 24 "
                  "25 26 "
                  "- - "
                  "27 28 "
                  "- - "
                  "32 33 "
                  "34 35 "),
        Connector("j", 5, # J5
                  "- - "
                  "71 69 "
                  "70 68 "
                  "67 62 "
                  "65 61 "
                  "- - "
                  "60 58 "
                  "59 57 "
                  "- - "
                  "56 54 "
                  "55 52 "
                  "- - "
                  "50 48 "
                  "49 47 "
                  "- - "
                  "45 43 "
                  "44 42 "
                  "- - "
                  "41 39 "
                  "40 38 "),
    ]

    def toolchain_program(self, products, name):
        tool = os.environ.get("OPENFPGALOADER", "openFPGALoader")
        with products.extract("{}.bit".format(name)) as bitstream_filename:
            subprocess.check_call([tool, "-b", "machXO2EVN", "-m", bitstream_filename])


# This is an older version of the board, that has an FPGA with less logic
# resources. It is otherwise the same.
class MachXO2_1200ZE_BreakoutPlatform(MachXO2_7000HE_BreakoutPlatform):
    device         = "LCMXO2-1200ZE"
    speed          = "1"


if __name__ == "__main__":
    from .test.blinky import *
    MachXO2_7000HE_BreakoutPlatform().build(Blinky(), do_program=True)



import os
import subprocess

from amaranth.build import *
from amaranth.vendor import GowinPlatform

from .resources import *


__all__ = ["TangNano9kPlatform"]


class TangNano9kPlatform(GowinPlatform):
    part          = "GW1NR-LV9QN88PC6/I5"
    family        = "GW1NR-9C"
    default_clk   = "clk27"
    resources     = [
        Resource("clk27", 0, Pins("52", dir="i"),
                 Clock(27e6), Attrs(IO_TYPE="LVCMOS33")),

        *ButtonResources(pins="3 4", invert=True,
                         attrs=Attrs(IO_TYPE="LVCMOS33")),

        *LEDResources(pins="10 11 13 14 15 16", invert=True,
                      attrs=Attrs(IO_TYPE="LVCMOS33")),

        UARTResource(0, rx="18", tx="17",
            attrs=Attrs(PULL_MODE="UP", IO_TYPE="LVCMOS33")),

        *SPIFlashResources(0,
            cs_n="60", clk="59", copi="61", cipo="62",
            attrs=Attrs(IO_TYPE="LVCMOS33")),

        *SDCardResources(0,
            clk="36", cmd="37", dat0="39", dat3="38", wp_n="-",
            attrs=Attrs(IO_TYPE="LVCMOS33")),

        Resource("lcd", 0,
            Subsignal("clk", Pins("35", dir="o")),
            Subsignal("hs", Pins("40", dir="o")),
            Subsignal("vs", Pins("34", dir="o")),
            Subsignal("de", Pins("33", dir="o")),
            Subsignal("r", Pins("75 74 73 72 71", dir="o")),
            Subsignal("g", Pins("70 69 68 57 56 55", dir="o")),
            Subsignal("b", Pins("54 53 51 42 41", dir="o")),
            Attrs(IO_TYPE="LVCMOS33", DRIVE=24)),

        Resource("lcd_backlight", 0, Pins("86", dir="o"),
                 Attrs(IO_TYPE="LVCMOS33")),

        Resource("hdmi", 0,
             Subsignal("clk", DiffPairs(p="69", n="68", dir="o")),
             Subsignal("d", DiffPairs(p="71 73 75", n="70 72 74", dir="o")),
             Attrs(IO_TYPE="LVCMOS33")),
    ]
    connectors = []

    def toolchain_prepare(self, fragment, name, **kwargs):
        overrides = {
            "add_options":
                "set_option -use_mspi_as_gpio 1 -use_sspi_as_gpio 1",
            "gowin_pack_opts":
                "--sspi_as_gpio --mspi_as_gpio"
        }
        return super().toolchain_prepare(fragment, name, **overrides, **kwargs)

    def toolchain_program(self, products, name):
        with products.extract("{}.fs".format(name)) as bitstream_filename:
            subprocess.check_call(["openFPGALoader", "-b", "tangnano9k", bitstream_filename])


if __name__ == "__main__":
    from .test.blinky import *
    TangNano9kPlatform().build(Blinky(), do_program=True)



import itertools

from amaranth import *
from amaranth.build import ResourceError
from amaranth.lib import io


__all__ = ["Blinky"]


class Blinky(Elaboratable):
    def elaborate(self, platform):
        m = Module()

        def get_all_resources(name):
            resources = []
            for number in itertools.count():
                try:
                    resources.append(platform.request(name, number, dir="-"))
                except ResourceError:
                    break
            return resources

        rgb_leds = [res for res in get_all_resources("rgb_led")]
        leds     = [io.Buffer("o", res) for res in get_all_resources("led")]
        leds.extend([io.Buffer("o", led.r) for led in rgb_leds])
        leds.extend([io.Buffer("o", led.g) for led in rgb_leds])
        leds.extend([io.Buffer("o", led.b) for led in rgb_leds])
        buttons  = [io.Buffer("i", res) for res in get_all_resources("button")]
        switches = [io.Buffer("i", res) for res in get_all_resources("switch")]

        m.submodules += leds + buttons + switches

        inverts  = [0 for _ in leds]
        for index, button in zip(itertools.cycle(range(len(inverts))), buttons):
            inverts[index] ^= button.i
        for index, switch in zip(itertools.cycle(range(len(inverts))), switches):
            inverts[index] ^= switch.i

        clk_freq = platform.default_clk_frequency
        timer = Signal(range(int(clk_freq//2)), init=int(clk_freq//2) - 1)
        flops = Signal(len(leds))

        m.d.comb += Cat(led.o for led in leds).eq(flops ^ Cat(inverts))
        with m.If(timer == 0):
            m.d.sync += timer.eq(timer.init)
            m.d.sync += flops.eq(~flops)
        with m.Else():
            m.d.sync += timer.eq(timer - 1)

        return m



import os
import subprocess

from amaranth.build import *
from amaranth.vendor import GowinPlatform

from .resources import *


__all__ = [
    "TangPrimer20kPlatform",
    "TangPrimer20kLitePlatform",
    "TangPrimer20kDockPlatform",
]


class TangPrimer20kPlatform(GowinPlatform):
    part          = "GW2A-LV18PG256C8/I7"
    family        = "GW2A-18C"
    default_clk   = "clk27"
    resources     = [
        Resource("clk27", 0, Pins("H11", dir="i"),
                 Clock(27e6), Attrs(IO_TYPE="LVCMOS33")),

        UARTResource(0, rx="T13", tx="M11",
                     attrs=Attrs(PULL_MODE="UP", IO_TYPE="LVCMOS33")),

        *SPIFlashResources(0, cs_n="M9", clk="L10", cipo="P10", copi="R10",
                           attrs=Attrs(IO_TYPE="LVCMOS33")),

        *SDCardResources(0, clk="N10", cmd="R14", cd="D15",
                         dat0="M8", dat1="M7", dat2="M10", dat3="N11",
                         attrs=Attrs(IO_TYPE="LVCMOS33")),

        DDR3Resource(0, rst_n="B9", clk_p="J1", clk_n="J3", clk_en="J2",
                     cs_n="P5", we_n="L2", ras_n="R4", cas_n="R6",
                     a="F7 A4 D6 F8 C4 E6 B1 D8 A5 F9 K3 B7 A3 C8",
                     ba="H4 D3 H5", dqs_p="G2 J5", dqs_n="G3 K6",
                     dq="G5 F5 F4 F3 E2 C1 E1 B3 M3 K4 N2 L1 P4 H3 R1 M2",
                     dm="G1 K5", odt="R3",
                     attrs=Attrs(IO_TYPE="SSTL15"),
                     diff_attrs=Attrs(IO_TYPE="SSTL15D")),

        # ZIF connector on core board
        Resource("spi_lcd", 0,
                 Subsignal("bl",   PinsN("P12", dir="o")),
                 Subsignal("rst",  PinsN("G13", dir="o")),
                 Subsignal("rs",   Pins("J13", dir="o")),
                 Subsignal("cs",   PinsN("C16", dir="o")),
                 Subsignal("clk",  Pins("F12", dir="o")),
                 Subsignal("copi", Pins("L15", dir="o")),
                 Attrs(IO_TYPE="LVCMOS33")),
    ]

    connectors = [
        # GND, 5V, 3V3 are power pins
        # V01 and V7 are IO bank voltages, tied to 3V3 by R5 and R9 (resp.)
        Connector("sodimm", 0,
                  #     GND GND 5V  5V  5V  5V  GND GND NC  (  1- 10)
                  " T13  -   -   -   -   -   -   -   -   - "
                  #     NC  GND GND     NC  NC  NC  GND GND ( 11- 20)
                  " M11  -   -   -  T10  -   -   -   -   - "
                  # NC  3V3 NC  3V3 GND GND                 ( 21- 30)
                  "  -   -   -   -   -   -  T6  R16 P6  P16"
                  # GND GND                 GND GND         ( 31- 40)
                  "  -   -  T7  P16 R8  N15  -   -  T8  N16"
                  #         GND                 GND GND     ( 41- 50)
                  " P8  N14  -  L16 T9  L14 P9   -   -  K15"
                  #             GND GND                 GND ( 51- 60)
                  " P11 K14 T11  -   -  K16 R11 J15 T12  - "
                  # GND                 GND                 ( 61- 70)
                  "  -  H16 R12 H14 P13  -  R13 G16 T14 H15"
                  # GND GND                                 ( 71- 80)
                  "  -   -  M15 L13 M14 K11 F13 K12 G12 K13"
                  #     NC                  NC  NC          ( 81- 90)
                  " T15  -  J16 H13 J14 J12  -   -  G14 H12"
                  #         NC  NC                  NC  NC  ( 91-100)
                  " G15 G11  -   -  F14 B10 F16 A13  -   - "
                  #     NC      NC  NC  NC      NC      NC  (101-110)
                  "E15  -   D14  -   -   -  A15  -  B14  - "
                  # NC  NC      NC      NC  NC  NC      NC  (111-120)
                  "  -   -  A14  -  B13  -   -   -  C12  - "
                  #     NC      NC      NC  GND GND         (121-130)
                  " B12  -  A12  -  C11  -   -   -  B11 E16"
                  #         GND GND         NC  GND GND     (131-140)
                  " A11 F15  -   -  C10 C13  -   -   -  D16"
                  # NC          GND GND                 GND (141-150)
                  "  -  E14 B8   -   -  C9  C6  A9  A7   - "
                  # GND             GND GND                 (151-160)
                  "  -  L12 A6  J11  -   -  C7  E9  D7  E8 "
                  # GND GND     V01     V01 GND GND     V7  (161-170)
                  "  -   -  T2   -  T3   -   -   -  T4   - "
                  #     GND GND V7              GND GND     (171-180)
                  " T5   -   -   -  N6  F10 N7   -   -  D11"
                  #             GND GND     NC  NC  GND GND (181-190)
                  " N9  D10 R9   -   -  E10  -   -   -   - "
                  #                 GND GND NC      NC      (191-200)
                  " N8  R7  L9  P7   -   -   -  M6   -  L8 "
                  # NC  NC  NC  NC                          (201-204)
                  "  -   -   -   - "
                  )
    ]

    def toolchain_prepare(self, fragment, name, **kwargs):
        overrides = {
            "add_options":
                "set_option -use_mspi_as_gpio 1 -use_sspi_as_gpio 1"
                " -use_ready_as_gpio 1 -use_done_as_gpio 1",
            "gowin_pack_opts":
                "--mspi_as_gpio --sspi_as_gpio --ready_as_gpio --done_as_gpio",
        }
        return super().toolchain_prepare(fragment, name, **overrides, **kwargs)

    def toolchain_program(self, products, name):
        with products.extract("{}.fs".format(name)) as bitstream_filename:
            subprocess.check_call(
                ["openFPGALoader", "-b", "tangprimer20k", bitstream_filename])


class TangPrimer20kLitePlatform(TangPrimer20kPlatform):
    resources = TangPrimer20kPlatform.resources + [
        *ButtonResources(pins={0: "T10"}, invert=True,
                         attrs=Attrs(IO_TYPE="LVCMOS33")),
        *ButtonResources(pins={1: "T2"}, invert=True,
                         attrs=Attrs(IO_TYPE="LVCMOS15")),

        *SwitchResources(pins="D7 C7",
                         attrs=Attrs(PULL_MODE="NONE", IO_TYPE="LVCMOS15")),
    ]

    connectors = TangPrimer20kPlatform.connectors + [
        # top left, top right, bottom left, bottom right
        Connector("pmod", 0, "L8  P7  E10 D11 - - M6  R7  D10 F10 - -"),
        Connector("pmod", 1, "T6  T7  T8  T9  - - P6  R8  P8  P9  - -"),
        Connector("pmod", 2, "F15 D16 C9  L12 - - E16 E14 A9  J11 - -"),
        Connector("pmod", 3, "R16 P16 N16 L16 - - P15 N15 N14 L14 - -"),

        # 0 on top, 1 on bottom, odd/even rows
        Connector("gpio", 0,
                  " -   -  N6  N7  B11 A12 L9  N8  R9  N9 " # ( 1-10)
                  "A6  A7  C6  B8  C10  -  A11 C11 B12 C12" # (11-20)
                  "B13 A14 B14 A15 D14 E15 F16 F14 G15 G14" # (21-30)
                  "J14 J16 G12 F13 M14 M15 T14 R13 P13 R12" # (31-40)
                  ),
        Connector("gpio", 1,
                  " -   -   -   -   -   -  T5   -  T3  T4 " # ( 1-10)
                  " -   -  E9  E8  T15 C13 T13 M11 B10 A13" # (11-20)
                  "H12 G11 H13 J12 K12 K13 L13 K11 R11 T12" # (21-30)
                  "P11 T11 G16 H15 H16 H14 K16 J15 K15 K14" # (31-40)
                  ),
    ]


class TangPrimer20kDockPlatform(TangPrimer20kPlatform):
    resources = TangPrimer20kPlatform.resources + [
        *LEDResources(pins="C13 A13 N16 N14 L14 L16", invert=True,
                      attrs=Attrs(IO_TYPE="LVCMOS33")),

        # WS2812 RGB LED
        Resource("ws2812", 0, Pins("T9", dir="o"),
                 Attrs(IO_TYPE="LVCMOS33")),

        *ButtonResources(pins={0: "T10"}, invert=True,
                         attrs=Attrs(IO_TYPE="LVCMOS33")),
        *ButtonResources(pins={1: "T3", 2: "T2", 3: "D7", 4: "C7"}, invert=True,
                         attrs=Attrs(IO_TYPE="LVCMOS15")),

        *SwitchResources(pins={0: "B10"},
                         attrs=Attrs(PULL_MODE="DOWN", IO_TYPE="LVCMOS33")),
        *SwitchResources(pins={1: "E9", 2: "E8", 3: "T4", 4: "T5"},
                         attrs=Attrs(PULL_MODE="DOWN", IO_TYPE="LVCMOS15")),

        Resource("eth_clk50", 0, Pins("A9", dir="i"),
                 Clock(50e6), Attrs(IO_TYPE="LVCMOS33")),
        Resource("eth_rmii", 0,
                 Subsignal("rst",     PinsN("F10", dir="o")),
                 Subsignal("mdio",    Pins("F16", dir="io")),
                 Subsignal("mdc",     Pins("F14", dir="o")),
                 Subsignal("tx_en",   Pins("E16", dir="o")),
                 Subsignal("tx_data", Pins("D16 E14", dir="o")),
                 Subsignal("rx_crs",  Pins("M6",  dir="i")),
                 Subsignal("rx_er",   Pins("L8" , dir="i")),
                 Subsignal("rx_data", Pins("F15 C9", dir="i")),
                 Attrs(IO_TYPE="LVCMOS33")),

        ULPIResource(0, data="G11 H12 J12 H13 T14 R13 P13 R12",
                     clk="T15", dir="K12", nxt="K13", stp="K11",
                     rst="F10", rst_invert=True,
                     attrs=Attrs(PULL_MODE="NONE", IO_TYPE="LVCMOS33")),

        Resource("hdmi", 0,
                 Subsignal("clk", DiffPairs(p="G16", n="H15", dir="o")),
                 Subsignal("d",   DiffPairs(p="H14 J15 K14", n="H16 K16 K15", dir="o")),
                 Subsignal("hdp", Pins("J11", dir="i"),
                           Attrs(IO_TYPE="LVCMOS33")),
                 Subsignal("cec", Pins("L12", dir="io"),
                           Attrs(IO_TYPE="LVCMOS33")),
                 Subsignal("sda", Pins("F14", dir="io"),
                           Attrs(IO_TYPE="LVCMOS33")),
                 Subsignal("scl", Pins("F16", dir="io"),
                           Attrs(IO_TYPE="LVCMOS33")),
                 Attrs(PULL_MODE="NONE")),

        # labeled DISPLAY or RGB LCD
        Resource("lcd", 0,
                 Subsignal("bl",  Pins("E10", dir="o")),
                 Subsignal("clk", Pins("R9",  dir="o")),
                 Subsignal("de",  Pins("E15", dir="o")),
                 Subsignal("hs",  PinsN("A15", dir="o")),
                 Subsignal("vs",  PinsN("D14", dir="o")),
                 Subsignal("r",   Pins("L9 N8 N9 N7 N6", dir="o")),
                 Subsignal("g",   Pins("D11 A11 B11 P7 R7 D10", dir="o")),
                 Subsignal("b",   Pins("B12 C12 B13 A14 B14", dir="o")),
                 Attrs(IO_TYPE="LVCMOS33")),

        # labeled CAM or DVP
        Resource("dvp", 0,
                 Subsignal("d",
                           Pins("T12 T11 P11 R11 M15 M14 J16 J14", dir="i")),
                 Subsignal("scl",  Pins("F14", dir="io")),
                 Subsignal("sda",  Pins("F16", dir="io")),
                 Subsignal("pclk", Pins("F13", dir="i")),
                 Subsignal("xclk", Pins("G12", dir="o")),
                 Subsignal("rst",  PinsN("L13", dir="o")),
                 Subsignal("vs",   Pins("G15", dir="i")),
                 Subsignal("pwdn", Pins("C10", dir="o")),
                 Subsignal("hs",   Pins("G14", dir="i")),
                 Attrs(IO_TYPE="LVCMOS33")),

        # labeled TP or CTP
        Resource("lcd_touch", 0,
                 Subsignal("sda", Pins("F14", dir="io")),
                 Subsignal("scl", Pins("F16", dir="io")),
                 Subsignal("int", Pins("C11", dir="i")),
                 Subsignal("rst", PinsN("A12", dir="o")),
                 Attrs(IO_TYPE="LVCMOS33")),

        # labeled GPIO or MIC ARRAY
        Resource("mic_i2s", 0,
                 Subsignal("sd",  Pins("P8 T8 R8 T7", dir="i")),
                 Subsignal("ws",  Pins("P6", dir="i")),
                 Subsignal("clk", Pins("T6", dir="i")),
                 Attrs(IO_TYPE="LVCMOS33")),
        Resource("mic_led", 0,
                 Subsignal("clk", Pins("P9", dir="o")),
                 Subsignal("dat", Pins("T9", dir="o")),
                 Attrs(IO_TYPE="LVCMOS33")),

        # 3.5mm audio via PT8211
        Resource("dac_i2s", 0,
                 Subsignal("sd",    Pins("P15", dir="o")),
                 Subsignal("ws",    Pins("P16", dir="o")),
                 Subsignal("clk",   Pins("N15", dir="o")),
                 Subsignal("pa_en", Pins("R16", dir="o")),
                 Attrs(PULL_MODE="UP", IO_TYPE="LVCMOS33"))
    ]

    connectors = TangPrimer20kPlatform.connectors + [
        # top left to top right
        Connector("pmod", 0, "B11 D11 N7  N8  - - A11 N6  N9  L9  - -"),
        Connector("pmod", 1, "E15 A15 A14 C12 - - D14 B14 B13 B12 - -"),
        Connector("pmod", 2, "P11 R11 M15 J16 - - T11 T12 M14 J14 - -"),
        Connector("pmod", 3, "P6  T7  P8  T9  - - T6  R8  T8  P9  - -"),
    ]


if __name__ == "__main__":
    from .test.blinky import *
    TangPrimer20kDockPlatform().build(Blinky(), do_program=True)



import os
import subprocess

from amaranth.build import *
from amaranth.vendor import LatticeECP5Platform
from .resources import *


__all__ = ["LogicbonePlatform", "Logicbone85FPlatform"]


class LogicbonePlatform(LatticeECP5Platform):
    name        = "Logicbone"
    device      = "LFE5UM5G-45F"
    package     = "BG381"
    speed       = "8"

    default_clk = "refclk"

    resources   = [
        Resource("refclk", 0, Pins("M19", dir="i"),
                 Clock(25e6), Attrs(IO_TYPE="LVCMOS18")),

        # SerDes connections.
        Resource("serdes", 0,
            Subsignal("rx", DiffPairs("Y5", "Y6")),
            Subsignal("tx", DiffPairs("W4", "W5")),
        ),
        Resource("serdes", 1,
            Subsignal("rx", DiffPairs("Y8", "Y7")),
            Subsignal("tx", DiffPairs("W8", "W9")),
        ),

        # VBUS Detection is on the USB-C PD controller for both ports.
        DirectUSBResource("usb", 0,
            d_p="B12", d_n="C12", pullup="C16",
            attrs=Attrs(IO_TYPE="LVCMOS33")
        ),
        DirectUSBResource("usb", 1,
            d_p="R1",  d_n="T1",  pullup="T2",
            attrs=Attrs(IO_TYPE="LVCMOS33")
        ),

        *LEDResources(pins="D16 C15 C13 B13", attrs=Attrs(IO_TYPE="LVCMOS33"), invert=True),

        *ButtonResources(pins="U2", attrs=Attrs(IO_TYPE="LVCMOS33"), invert=True),

        *SPIFlashResources(0,
            cs_n="R2", clk="U3", cipo="V2", copi="W2", wp_n="Y2", hold_n="W1",
            attrs=Attrs(IO_TYPE="LVCMOS33")
        ),

        *SDCardResources(0,
            clk="E11", cmd="D15", cd="D14",
            dat0="D13", dat1="E13", dat2="E15", dat3="E13",
            attrs=Attrs(IO_TYPE="LVCMOS33")
        ),

        Resource("eth_clk125", 0, Pins("A19", dir="i"),
                 Clock(125e6), Attrs(IO_TYPE="LVCMOS33")),
        Resource("eth_rgmii", 0,
            # Stolen for sys_reset usage on prototypes.
            # Subsignal("rst",     PinsN("U17", dir="o")),
            Subsignal("int",     Pins("B20", dir="i")),
            Subsignal("mdc",     Pins("D12", dir="o")),
            Subsignal("mdio",    Pins("B19", dir="io")),
            Subsignal("tx_clk",  Pins("A15", dir="o")),
            Subsignal("tx_ctl",  Pins("B15", dir="o")),
            Subsignal("tx_data", Pins("A12 A13 C14 A14", dir="o")),
            Subsignal("rx_clk",  Pins("B18", dir="i")),
            Subsignal("rx_ctl",  Pins("A18", dir="i")),
            Subsignal("rx_data", Pins("B17 A17 B16 A16", dir="i")),
            Attrs(IO_TYPE="LVCMOS33")
        ),

        Resource("ddr3", 0,
            Subsignal("rst",     PinsN("P1", dir="o")),
            Subsignal("clk",     DiffPairs("M4", "N5", dir="o"), Attrs(IO_TYPE="LVDS")),
            Subsignal("clk_en",  Pins("K4", dir="o")),
            Subsignal("cs",      PinsN("M3", dir="o")),
            Subsignal("we",      PinsN("E4", dir="o")),
            Subsignal("ras",     PinsN("L1", dir="o")),
            Subsignal("cas",     PinsN("M1", dir="o")),
            Subsignal("a",       Pins("D5 F4 B3 F3 E5 C3 C4 A5 A3 B5 G3 F5 D2 A4 D3 E3", dir="o")),
            Subsignal("ba",      Pins("B4 H5 N2", dir="o")),
            Subsignal("dqs",     DiffPairs("K2 H4", "J1 G5", dir="io"), Attrs(IO_TYPE="LVDS")),
            Subsignal("dq",      Pins("G2 K1 F1 K3 H2 J3 G1 H1 B1 E1 A2 F2 C1 E2 C2 D1", dir="io")),
            Subsignal("dm",      Pins("L4 J5", dir="o")),
            Subsignal("odt",     Pins("C5", dir="o")),
            Attrs(IO_TYPE="SSTL135_I")
        )
    ]
    connectors = [
        Connector("P8", 0, """
        -   -   C20 D19 D20 E19 E20 F19 F20 G20 -   -   -   -   -   -
        -   -   -   -   -   -   G19 H20 J20 K20 C18 D17 D18 E17 E18 F18
        F17 G18 E16 F16 G16 H16 J17 J16 H18 H17 J19 K19 J18 K18
        """),
        Connector("P9", 0, """
        -   -   -   -   -   -   -   -   -   -   -   A11 B11 A10 C10 A9
        B9  C11 A8  -   -   D9  C8  B8  A7  A6  B6  D8  C7  D7  C6  D6
        -   -   -   -   -   -   -   -   -   B10 E10 -   -   -
        """),
    ]

    def toolchain_prepare(self, fragment, name, **kwargs):
        overrides = dict(ecppack_opts="--compress --spimode qspi --freq 38.8")
        overrides.update(kwargs)
        return super().toolchain_prepare(fragment, name, **overrides)

    def toolchain_program(self, products, name):
        dfu_util = os.environ.get("DFU_UTIL", "dfu-util")
        with products.extract("{}.bit".format(name)) as bitstream_filename:
            subprocess.check_call([
                dfu_util, "-d", "1d50:615d", "-a", "0", "-R",
                "-D", bitstream_filename
            ])


class Logicbone85FPlatform(LogicbonePlatform):
    name        = "Logicbone (85F Variant)"
    device      = "LFE5UM5G-85F"


if __name__ == "__main__":
    import argparse
    from .test.blinky import Blinky

    parser = argparse.ArgumentParser()
    parser.add_argument("--variant", choices=("85", "45"), default="45",
        help="platform variant (default: %(default)s)")

    args = parser.parse_args()
    if args.variant == "45":
        platform = LogicbonePlatform()
    if args.variant == "85":
        platform = Logicbone85FPlatform()
    platform.build(Blinky(), do_program=True)



import os
import subprocess

from amaranth.build import *
from amaranth.vendor import XilinxPlatform
from .resources import *

"""
Example Usage:
    platform = CModS7_Platform(toolchain="Symbiflow")
    platform.build(Top(), do_program=True)

Supported programmer:
    openocd
"""

__all__ = ["CmodS7_Platform"]


class CmodS7_Platform(XilinxPlatform):
    device      = "xc7s25"
    package     = "csga225"
    speed       = "1"
    default_clk = "clk12"
    resources   = [
        Resource("clk12", 0, Pins("M9", dir="i"),
                 Clock(12e6), Attrs(IOSTANDARD="LVCMOS33")),

        *LEDResources(pins="E2 K1 J1 E1", attrs=Attrs(IOSTANDARD="LVCMOS33")),

        RGBLEDResource(0, r="F2", g="D3", b="F1", invert=True,
            attrs=Attrs(IOSTANDARD="LVCMOS33")),

        *ButtonResources(pins="D2 D1", attrs=Attrs(IOSTANDARD="LVCMOS33")),

        UARTResource(0,
            rx="K15", tx="L12",
            attrs=Attrs(IOSTANDARD="LVCMOS33")
        ),

        # Clock only via STARTUPE2 primitive
        *SPIFlashResources(0,
            cs_n="L11", clk="F5", copi="H14", cipo="H15", wp_n="J12",
            hold_n="K13",
            attrs=Attrs(IOSTANDARD="LVCMOS33")
        ),

        # One-wire interface to crypto authentication device
        # May not be populated on the board
        Resource("atsha204a", 0, Pins("D17", dir="io"),
            Attrs(IOSTANDARD="LVCMOS33"))
    ]
    connectors  = [
        Connector("pmod", 0, "J2 H2 H4 F3 - - H3 H1 G1 F4 - -"), # JA

        # Pin 24/25 are VCC and GND
        # Pin 32/33 are analog (XADC)
        # Pin 9-15 and 34-39 do not exist
        Connector("gpio", 0,
            """
            L1  M4  M3  N2  M2  P3  N3  P1  N1  -   -   -
            -   -   -   P14 P15 N13 N15 N14 M15 M14 L15 -
            -   L14 K14 J15 L13 M13 J11 -   -   -   -   -
            -   -   -   C5  A2  B2  B1  C1  B3  B4  A3  A4
            """),

        Connector("xadc", 0, {
            "vaux5_n":   "A13",
            "vaux5_p":   "A14",
            "vaux12_n":  "A11",
            "vaux12_p":  "A12"
        })
    ]

    def toolchain_program(self, products, name):
        with products.extract("{}.bit".format(name)) as bitstream_filename:
            subprocess.check_call(["openFPGALoader",
                "-c", "digilent",
                "--fpga-part", "xc7s25",
                "{}".format(bitstream_filename)
            ])


if __name__ == "__main__":
    from .test.blinky import *
    CmodS7_Platform().build(Blinky(), do_program=True)



from amaranth import *
from amaranth.build import *
from amaranth.vendor import IntelPlatform
from .resources import *

__all__ = ["MiSTPlatform"]


class MiSTPlatform(IntelPlatform):
    """MiST is the original Amiga/Atari ST retro FPGA board. It has been
    the inspiration for the MiSTer board.

    More information can be found on the wiki page:
    https://github.com/mist-devel/mist-board/wiki
    """
    device = "EP3C25" # Cyclone III 25K LEs
    package = "E144"
    speed = "C8"
    default_clk = "clk27"
    resources = [
        Resource(
            "clk27", 0, Pins("54", dir="i"),
            Clock(27e6), Attrs(io_standard="3.3-V LVTTL"),
        ),
        Resource(
            "clk27", 1, Pins("55", dir="i"),
            Clock(27e6), Attrs(io_standard="3.3-V LVTTL")),

        *LEDResources(
            pins="7",
            attrs=Attrs(io_standard="3.3-V LVTTL"),
        ),

        VGAResource(0,
            r="135 137 141 142 143 144",
            g="106 110 111 112 113 114",
            b="115 120 121 125 132 133",
            hs="119", vs="136",
            attrs=Attrs(io_standard="3.3-V LVTTL"),
        ),

        UARTResource(0,
            rx="31", tx="46",
            attrs=Attrs(io_standard="3.3-V LVTTL"),
        ),

        SPIResource(0,
            cs_n="127 91 90", # SS2/FPGA, SS3/OSD, SS4/SD_DIRECT
            clk="126", copi="88", cipo="105",
            attrs=Attrs(io_standard="3.3-V LVTTL"),
        ),

        SDRAMResource(0,
            clk="43", cs_n="59", we_n="66", ras_n="60", cas_n="64",
            ba="58 51", a="49 44 42 39 4 6 8 10 11 28 50 30 32",
            dq="83 79 77 76 72 71 69 68 86 87 98 99 100 101 103 104",
            dqm="67 85",
            attrs=Attrs(io_standard="3.3-V LVCMOS"),
        ),

        Resource("audio", 0,
            Subsignal("l", Pins("65", dir="o")),
            Subsignal("r", Pins("80", dir="o")),
            Attrs(io_standard="3.3-V LVTTL"),
        ),

        Resource("conf_data0", 0,
            Pins("13", dir="i"),
            Attrs(io_standard="3.3-V LVTTL"),
        ),
    ]
    connectors = []


if __name__ == "__main__":
    from .test.blinky import Blinky
    MiSTPlatform().build(Blinky(), do_program=False)
    print((
        "To use test bitstream on MiST; copy 'build/top.rbf' to file\n"
        "named 'core.rbf' on SD card so it will be loaded by MiST firmware"
    ))



import os
import subprocess

from amaranth.build import *
from amaranth.vendor import XilinxPlatform
from .resources import *


__all__ = ["RedPitaya14Platform"]


class RedPitaya14Platform(XilinxPlatform):
    device      = "xc7z010"
    package     = "clg400"
    speed       = "1"
    default_clk = "clk125"
    resources   = [
        Resource("clk125", 0,
            DiffPairs("U18", "U19", dir="i"), Clock(125e6), Attrs(IOSTANDARD="TMDS_33")),

        *LEDResources(
            pins="F16 F17 G15 H15 K14 G14 J15 J14",
            attrs=Attrs(IOSTANDARD="LVCMOS33")),

        Resource("daisy_io", 0,
            DiffPairs("T12", "U12", dir="i"), Attrs(IOSTANDARD="TMDS_33")),
        Resource("daisy_io", 1,
            DiffPairs("U14", "U15", dir="i"), Attrs(IOSTANDARD="TMDS_33")),
        Resource("daisy_io", 2,
            DiffPairs("P14", "R14", dir="i"), Attrs(IOSTANDARD="TMDS_33")),
        Resource("daisy_io", 3,
            DiffPairs("N18", "P19", dir="i"), Attrs(IOSTANDARD="TMDS_33")),
    ]
    connectors = [
            Connector("E1", 0, {
            # Outer Analog Header
            "dio0_p": "G17",
            "dio0_n": "G18",
            "dio1_p": "H16",
            "dio1_n": "H17",
            "dio2_p": "J18",
            "dio2_n": "H18",
            "dio3_p": "K17",
            "dio3_n": "K18",
            "dio4_p": "L14",
            "dio4_n": "L15",
            "dio5_p": "L16",
            "dio5_n": "L17",
            "dio6_p": "K16",
            "dio6_n": "J16",
            "dio7_p": "M14",
            "dio7_n": "M15",
        })
    ]

    def toolchain_program(self, products, name, **kwargs):
        with products.extract("{}.bit".format(name)) as bitstream_filename:
            subprocess.check_call(["openFPGALoader", "-c", "digilent_hs2", bitstream_filename])


if __name__ == "__main__":
    from .test.blinky import *
    RedPitaya14Platform().build(Blinky(), do_program=True)



import os
import argparse
import subprocess


from amaranth.build import *
from amaranth.vendor import LatticeMachXO2Platform
from .resources import *


__all__ = ["StepMXO2Platform"]


class StepMXO2Platform(LatticeMachXO2Platform):
    package     = "MG132"
    speed       = "4"
    default_clk = "clk12"
    device      = "LCMXO2-4000HC"
    resources   = [
        Resource("clk12", 0, Pins("C1", dir="i"), Attrs(IO_TYPE="LVCMOS33"),
                 Clock(12e6)),

        UARTResource(0, rx="A3", tx="A2", attrs=Attrs(IO_TYPE="LVCMOS33"),
                     role="dce"),

        *LEDResources(pins="N13 M12 P12 M11 P11 N10 N9 P9",
                      invert=True, attrs=Attrs(IO_TYPE="LVCMOS33")),

        RGBLEDResource(0, r="M2", g="N2", b="P2",
                       invert=True, attrs=Attrs(IO_TYPE="LVCMOS33")),
        RGBLEDResource(1, r="M3", g="N3", b="P4",
                       invert=True, attrs=Attrs(IO_TYPE="LVCMOS33")),

        *ButtonResources(pins="L14 M13 M14 N14", invert=True,
                         attrs=Attrs(IO_TYPE="LVCMOS33")),

        *SwitchResources(pins="M7 M8 M9 M10",
                         attrs=Attrs(IO_TYPE="LVCMOS33")),

        Display7SegResource(0,
            a="A10", b="C11", c="F2", d="E1", e="E2", f="A9", g="B9", dp="F1",
            attrs=Attrs(IO_TYPE="LVCMOS33")
        ),
        Display7SegResource(1,
            a="C12", b="B14", c="J1", d="H1", e="H2", f="B12", g="A11",
            dp="K1", attrs=Attrs(IO_TYPE="LVCMOS33")
        ),
        Resource("display_7seg_ctrl", 0,
            Subsignal("en", Pins("C9 A12", invert=True, dir="o")),
            Attrs(IO_TYPE="LVCMOS33")
        )
    ]
    connectors  = [
        # Special pins for MachXO2 hard IP.
        # I2C: 2- SCL, 3- SDA
        # SPI: 25- CS_N, 24- CLK, 22- COPI, 23- CIPO
        Connector("gpio", 0,
                  # Left side of the board
                  # 1  2  3  4  5  6  7  8  9 10 11 12 13 14 15 16 17 18 19 20
                  " - C8 B8 E3 F3 G3 H3 I2 I3 K2 K3 L3 N5 P6 N6 P7 N7 P8 N8  -"
                  # Right side of the board
                  # 21  22 23 24 25  26  27  28  29  30  31  32  33  34  35  36  37  38  39 40
                  "  - P13 N4 M4 P3 J12 K13 K14 K12 J14 J13 H12 G14 G13 F14 F13 G12 F12 E12  -"
        ),
    ]

    # This board doesn't have an integrated programmer; the board's MCU
    # implements a mass-storage device. You copy the JED file to it.

    # Workaround a STEP-MXO2-LPC parsing bug in the bootloader, as it doesn't
    # like JED files processed by ddtcmd. Copy the original JED file unchanged
    # to {{name}}-lpc.jed. Prefer this file.
    @property
    def command_templates(self):
        templates = super().command_templates
        cp_template = r"""{%- if syntax == "sh" -%}
                                cp {{name}}_impl/{{name}}_impl.jed {{name}}-lpc.jed
                          {%- else -%}
                                copy {{name}}_impl\{{name}}_impl.jed {{name}}-lpc.jed
                          {%- endif -%}
                       """

        if self.family == "machxo2":
            if self.toolchain == "Diamond":
                return templates + [cp_template]
            if self.toolchain == "Trellis":
                return self.templates
            assert False
        assert False


if __name__ == "__main__":
    from .test.blinky import *

    parser = argparse.ArgumentParser()
    parser.add_argument('toolchain', nargs="?", choices=["Trellis", "Diamond"], default="Diamond")
    args = parser.parse_args()

    StepMXO2Platform(toolchain=args.toolchain).build(Blinky(), do_program=False)



import subprocess
from amaranth.vendor import GowinPlatform
from amaranth.build import *
from .resources import *


class TangNanoPlatform(GowinPlatform):
    part        = "GW1N-LV1QN48C6/I5"
    family      = "GW1N-1"
    default_clk = "OSC"
    osc_frequency = 24_000_000
    resources   = [
        # This clock is shared with the USB-JTAG MCU and stops when the USB bus is suspended.
        # It probably should not be used, but is included for completeness.
        Resource("clk24", 0, Pins("35", dir="i"), Clock(24_000_000),
                 Attrs(IO_TYPE="LVCMOS33")),

        RGBLEDResource(0, r="18", g="16", b="17",
                       attrs=Attrs(IO_TYPE="LVCMOS33")),

        *ButtonResources(pins="15 14", invert=True,
                         attrs=Attrs(IO_TYPE="LVCMOS33")),

        UARTResource(0, tx="9", rx="8",
                     attrs=Attrs(IO_TYPE="LVCMOS33", PULL_MODE="UP")),

        *SPIFlashResources(0,
            cs_n="19", clk="20", copi="22", cipo="23", wp_n="24", hold_n="25",
            attrs=Attrs(IO_TYPE="LVCMOS33")),

        Resource("lcd", 0,
            Subsignal("clk", Pins("11", dir="o")),
            Subsignal("hs", Pins("10", dir="o")),
            Subsignal("vs", Pins("46", dir="o")),
            Subsignal("de", Pins("5", dir="o")),
            Subsignal("r", Pins("27 28 29 30 31", dir="o")),
            Subsignal("g", Pins("32 33 34 38 39 40", dir="o")),
            Subsignal("b", Pins("41 42 43 44 45", dir="o")),
            Attrs(IO_TYPE="LVCMOS33")),

        Resource("lcd_backlight", 0, Pins("47", dir="o"),
                 Attrs(IO_TYPE="LVCMOS33")),
    ]
    connectors  = []

    def toolchain_prepare(self, fragment, name, **kwargs):
        overrides = {
            "add_options":
                "set_option -use_mspi_as_gpio 1 -use_sspi_as_gpio 1",
            "gowin_pack_opts":
                "--sspi_as_gpio --mspi_as_gpio"
        }
        return super().toolchain_prepare(fragment, name, **overrides, **kwargs)

    def toolchain_program(self, products, name):
        with products.extract("{}.fs".format(name)) as bitstream_filename:
            subprocess.check_call(["openFPGALoader", "-b", "tangnano",
                                   bitstream_filename])


if __name__ == "__main__":
    from .test.blinky import *
    TangNanoPlatform().build(Blinky(), do_program=True)



import os
import subprocess

from amaranth.build import *
from amaranth.vendor import IntelPlatform
from .resources import *


__all__ = ["DE0NanoPlatform"]


class DE0NanoPlatform(IntelPlatform):
    device      = "EP4CE22" # Cyclone IV 22K LEs
    package     = "F17"   # FBGA-17
    speed       = "C6"
    default_clk = "clk50"
    resources   = [
        Resource("clk50", 0, Pins("R8", dir="i"),
                 Clock(50e6), Attrs(io_standard="3.3-V LVCMOS")),

        *LEDResources(
            pins="A15 A13 B13 A11 D1 F3 B1 L3",
            attrs=Attrs(io_standard="3.3-V LVCMOS")),
        *ButtonResources(
            pins="J15 E1", invert=True,
            attrs=Attrs(io_standard="3.3-V LVCMOS")),
        *SwitchResources(
            pins="M1 T8 B9 M15",
            attrs=Attrs(io_standard="3.3-V LVCMOS")),

        SDRAMResource(0,
            clk="R4", cke="L7", cs_n="P6", we_n="C2", ras_n="L2", cas_n="L1",
            ba="M7 M6", a="P2 N5 N6 M8 P8 T7 N8 T6 R1 P1 N2 N1 L4",
            dq="G2 G1 L8 K5 K2 J2 J1 R7 T4 T2 T3 R3 R5 P3 N3 K1", dqm="R6 T5",
            attrs=Attrs(io_standard="3.3-V LVCMOS")),
        
        I2CResource(0, scl="F2", sda="F1",
                    attrs=Attrs(io_standard="3.3-V LVCMOS")),
        
        Resource("adxl345", 0,
            Subsignal("scl",         Pins("F2"), Attrs(IOSTANDARD="I2C")),
            Subsignal("sda",         Pins("F1"), Attrs(IOSTANDARD="I2C")),
            Subsignal("G_sens_int",  Pins("M2", dir='i')),
            Subsignal("G_sens_cn_n", Pins("G5")),
            Attrs(io_standard="3.3-V LVCMOS"),
        ),
        SPIResource("spi_adc", 0, role="controller",
            cs_n="A10", clk="B14", copi="A9", cipo="B10",
            attrs=Attrs(io_standard="3.3-V LVCMOS"),
        ),
    ]
    connectors  = [
        Connector("gpio", 0,
            "A8  D3  B8  C3  A2  A3  B3  B4  A4  B5 "
            "-   -   A5  D5  B6  A6  B7  D6  A7  C6 "
            "C8  E6  E7  D8  E8  F8  F9  E9  -   -  "
            "C9  D9  E11 E10 C11 B11 A12 D11 D12 B12"),

        Connector("gpio", 1,
            "T9  F13 R9  T15 T14 T13 R13 T12 R12 T11"
            "-   -   T10 R11 P11 R10 N12 P9  N9  N11" 
            "L16 K16 R16 L15 P15 P16 R14 N16 -   -  "
            "N15 P14 L14 N14 M10 L13 J16 K15 J13 J14"),
        
        Connector("gpio", 2,
            "-   E15 E16 M16 A14 B16 C14 C16 C15" 
            "D16 D15 D14 F15 F16 F14 G16 G15"),
    ]

    def toolchain_program(self, products, name):
        quartus_pgm = os.environ.get("QUARTUS_PGM", "quartus_pgm")
        with products.extract("{}.sof".format(name)) as bitstream_filename:
            subprocess.check_call([quartus_pgm, "--haltcc", "--mode", "JTAG",
                                   "--operation", "P;" + bitstream_filename])


if __name__ == "__main__":
    from .test.blinky import Blinky
    DE0NanoPlatform().build(Blinky(), do_program=True)



import os
import subprocess

from amaranth.build import *
from amaranth.vendor import LatticeICE40Platform
from amaranth_boards.resources import *


__all__ = ["UpduinoV3Platform"]


class UpduinoV3Platform(LatticeICE40Platform):
    device      = "iCE40UP5K"
    package     = "SG48"
    default_clk = "SB_HFOSC"
    hfosc_div   = 0
    resources   = [
        # Solder the OSC jumper to connect the onboard oscillator to pin 20.
        # Note that this overlaps with the QSPI pins.
        Resource("clk12", 0,
            Pins("20", dir="i"),
            Clock(12e6), Attrs(IO_STANDARD="SB_LVCMOS")),

        RGBLEDResource(0, 
            r="41", g="39", b="40", invert=True,
            attrs=Attrs(IO_STANDARD="SB_LVCMOS")),

        # To use QSPI mode, solder the appropriate jumpers.
        *SPIFlashResources(0,
            cs_n="16", clk="15", cipo="17", copi="14", wp_n="10", hold_n="20",
            attrs=Attrs(IO_STANDARD="SB_LVCMOS")
        ),
    ]
    connectors  = [
        # "Left" row of header pins (JP5 on the schematic)
        Connector("j", 0, "41 39 40 - - - 23 25 26 27 32 35 31 37 34 43 36 42 38 28"),
        # "Right" row of header pins (JP6 on the schematic)
        Connector("j", 1, "20 10 - - 12 21 13 19 18 11 9 6 44 4 3 48 45 47 46 2")
    ]

    def toolchain_program(self, products, name):
        iceprog = os.environ.get("ICEPROG", "iceprog")
        with products.extract("{}.bin".format(name)) as bitstream_filename:
            subprocess.check_call([iceprog, bitstream_filename])


if __name__ == "__main__":
    from amaranth_boards.test.blinky import *
    UpduinoV3Platform().build(Blinky(), do_program=True)



import os
import subprocess

from amaranth.build import *
from amaranth.vendor import LatticeICE40Platform
from .resources import *


__all__ = ["ICEBreakerBitsyPlatform"]


class ICEBreakerBitsyPlatform(LatticeICE40Platform):
    device      = "iCE40UP5K"
    package     = "SG48"
    default_clk = "clk12"
    resources   = [
        Resource("clk12", 0, Pins("35", dir="i"),
                 Clock(12e6), Attrs(GLOBAL=True, IO_STANDARD="SB_LVCMOS")),

        DirectUSBResource(0, d_p="42", d_n="38", pullup="37",
            attrs=Attrs(IO_STANDARD="SB_LVCMOS")),

        *SPIFlashResources(0,
            cs_n="16", clk="15", copi="14", cipo="17", wp_n="18", hold_n="19",
            attrs=Attrs(IO_STANDARD="SB_LVCMOS")
        ),

        RGBLEDResource(0, r="39", g="40", b="41", invert=True,
            attrs=Attrs(IO_STANDARD="SB_LVCMOS")),
        *LEDResources(pins="25 6", invert=True, attrs=Attrs(IO_STANDARD="SB_LVCMOS")),
        Resource("led_r", 0, PinsN("25", dir="o"), Attrs(IO_STANDARD="SB_LVCMOS")),
        Resource("led_g", 0, PinsN("6", dir="o"), Attrs(IO_STANDARD="SB_LVCMOS")),

        *ButtonResources(pins="2", invert=True, attrs=Attrs(IO_STANDARD="SB_LVCMOS")),
    ]
    connectors = [
        Connector("edge", 0,  # Pins bottom P0 - P12,
            {"0":"47",  "1":"44",  "2":"48",  "3":"45",  "4": "4",  "5": "3",
             "6": "9",  "7":"10",  "8":"11",  "9":"12", "10":"21", "11":"13",
            "12":"20", "13":"25", "14":"23", "15":"27", "16":"26", "17":"28",
            "18":"31", "19":"32", "20":"34", "21":"36", "22":"43", "23":"46"}
        ),
        Connector("pmod", 1, " 0  2  4  6 - -  1  3  5  7 - -", conn=("edge", "0")), # PMOD 1
        Connector("pmod", 2, "22 19 16 17 - - 21 18 15 20 - -", conn=("edge", "0")), # PMOD 2
        Connector("pmod", 3, "14  9 11  8 - - 13 10 12 23 - -", conn=("edge", "0"))  # PMOD 3
    ]

    def toolchain_program(self, products, name, run_vid=None, run_pid=None, dfu_vid="1d50", dfu_pid="6146", reset=True):
        dfu_util = os.environ.get("DFU_UTIL", "dfu-util")

        # Construct the device runtime and DFU vid pid string
        dev_str = ""
        if run_vid or run_pid:
            dev_str = "{}:{}".format(run_vid or "", run_pid or "")
        dev_str += ",{}:{}".format(dfu_vid or "", dfu_pid or "")

        # Construct the argument list for dfu-util
        args = [dfu_util, "-d", dev_str, "-a", "0"]
        if reset: args.append("-R")
        args.append("-D")

        # Run dfu-util
        with products.extract("{}.bin".format(name)) as bitstream_filename:
            args.append(bitstream_filename)
            subprocess.check_call(args)


if __name__ == "__main__":
    from .test.blinky import *
    ICEBreakerBitsyPlatform().build(Blinky(), do_program=True)



import os
import subprocess
import textwrap
import unittest

from amaranth.build import *
from amaranth.vendor import XilinxPlatform
from .resources import *


__all__ = ["NitefuryIIPlatform", "LitefuryPlatform"]


class _BasePlatform(XilinxPlatform):
    speed       = "2"
    default_clk = "clk200"

    resources   = [
        Resource("clk200", 0, DiffPairs(p="J19", n="H19", dir="i"),
                 Clock(200e6), Attrs(IOSTANDARD="DIFF_SSTL15")),
        # 4 Programmable LEDs
        *LEDResources(pins="G3 H3 G4 H4", attrs=Attrs(IOSTANDARD="LVCMOS33")),
        # SPIFlash
        *SPIFlashResources(0,
            cs_n="T19", clk="L16", copi="P22", cipo="R22", wp_n="P21", hold_n="R21",
            attrs=Attrs(IOSTANDARD="LVCMOS33")
        ),
        # 1 M.2 indicator LED
        Resource("m2led", 0, Pins("M1"), Attrs(IOSTANDARD="LVCMOS33")),
        # PCIE
        Resource("pcie", 0,
            Subsignal("clkreq", PinsN("G1", dir="o"), Attrs(IOSTANDARD="LVCMOS33")),
            Subsignal("rst",    PinsN("J1", dir="o"), Attrs(IOSTANDARD="LVCMOS33", PULLUP=1)),
            Subsignal("clk",    DiffPairs(p="F6", n="E6", dir="i"), Attrs(IOSTANDARD="DIFF_SSTL15")),
            Subsignal("rx",     DiffPairs(p="B10 B8 D11 D9", n="A10 A8 C11 C9", dir="i")),
            Subsignal("tx",     DiffPairs(p="B6 B4 D5 D7", n="A6 A4 C5 C7", dir="o")),
        ),
        # DDR
        Resource("ddr3", 0,
            Subsignal("rst",    PinsN("K16", dir="o"), Attrs(IOStandard="LVCMOS15")),
            Subsignal("clk",    DiffPairs(p="K17", n="J17", dir="o")),
            Subsignal("clk_en", Pins("H22", dir="o")),
            # Subsignal("cs",     PinsN("U8", dir="o")),
            Subsignal("we",     PinsN("L16", dir="o")),
            Subsignal("ras",    PinsN("H20", dir="o")),
            Subsignal("cas",    PinsN("K18", dir="o")),
            Subsignal("a",      Pins("M15 L21 M16 L18 K21 M18 M21 N20 M20 N19 J21 M22 K22 N18 N22 J22", dir="o")),
            Subsignal("ba",     Pins("L19 J20 L20", dir="o")),
            Subsignal("dqs",    DiffPairs(p="F18 B21", n="E18 A21", dir="io"),
                                Attrs(IOSTANDARD="DIFF_SSTL135")),
            Subsignal("dq",     Pins("D19 B20 E19 A20 F19 C19 F20 C18 E22 G21 D20 E21 C22 D21 B22 D22", dir="io"),
                                Attrs(IN_TERM="UNTUNED_SPLIT_50")),
            Subsignal("dm",     Pins("A19 G22", dir="o")),
            Subsignal("odt",    Pins("K19", dir="o")),
            Attrs(IOSTANDARD="SSTL15", SLEW="FAST"),
        ),
    ]
    connectors  = []

    def toolchain_prepare(self, fragment, name, **kwargs):
        overrides = {
            "script_before_bitstream":
                """
                set_property BITSTREAM.CONFIG.SPI_BUSWIDTH 4 [current_design]
                set_property BITSTREAM.CONFIG.CONFIGRATE 16 [current_design]
                set_property BITSTREAM.GENERAL.COMPRESS TRUE [current_design]
                """,
            "script_after_bitstream":
                "write_cfgmem -force -format bin -interface spix4 -size 16 "
                "-loadbit \"up 0x0 {name}.bit\" -file {name}.bin".format(name=name),
            "add_constraints":
                """
                set_property INTERNAL_VREF 0.675 [get_iobanks 34]
                set_property CFGBVS VCCO [current_design]
                set_property CONFIG_VOLTAGE 3.3 [current_design]
                """
        }
        return super().toolchain_prepare(fragment, name, **overrides, **kwargs)

    def toolchain_program(self, product, name, *, programmer="openfpgaloader", flash=True):
        assert programmer in ("vivado", "openfpgaloader")

        if programmer == "vivado":
            if flash:
                # It does not appear possible to reset the FPGA via TCL after
                # flash programming.
                with product.extract("{}.bin".format(name)) as bitstream_filename:
                    cmd = textwrap.dedent("""
                        open_hw_manager
                        connect_hw_server
                        open_hw_target
                        current_hw_device [lindex [get_hw_devices xc7a*] 0]]
                        create_hw_cfgmem -hw_device [current_hw_device] s25fl256sxxxxxx0-spi-x1_x2_x4
                        set_property PROGRAM.FILES {{{}}} [current_hw_cfgmem]
                        set_property PROGRAM.ADDRESS_RANGE  {{use_file}} [current_hw_cfgmem]
                        set_property PROGRAM.BLANK_CHECK  1 [current_hw_cfgmem]
                        set_property PROGRAM.ERASE  1 [current_hw_cfgmem]
                        set_property PROGRAM.CFG_PROGRAM  1 [current_hw_cfgmem]
                        set_property PROGRAM.VERIFY  1 [current_hw_cfgmem]
                        create_hw_bitstream -hw_device [current_hw_device] [get_property PROGRAM.HW_CFGMEM_BITFILE [current_hw_device]]
                        program_hw_devices
                        program_hw_cfgmem
                        close_hw_manager
                        puts "Vivado TCL cannot reset boards. Reset or power-cycle your board now."
                    """).format(bitstream_filename).encode("utf-8")
                    subprocess.run(["vivado", "-nolog", "-nojournal", "-mode", "tcl"], input=cmd, check=True)
            else:
                with product.extract("{}.bit".format(name)) as bitstream_filename:
                    cmd = textwrap.dedent("""
                        open_hw_manager
                        connect_hw_server
                        open_hw_target
                        current_hw_device [lindex [get_hw_devices] 0]
                        set_property PROGRAM.FILE {{{}}} [current_hw_device]
                        program_hw_devices
                        close_hw_manager
                    """).format(bitstream_filename).encode("utf-8")
                    subprocess.run(["vivado", "-nolog", "-nojournal", "-mode", "tcl"], input=cmd, check=True)
        else:
            # openfpgaloader
            openfpgaloader = os.environ.get("OPENFPGALOADER", "openFPGALoader")
            with product.extract("{}.bin".format(name)) as fn:
                # TODO: @timkpaine has digilent_hs3 cable
                subprocess.check_call([openfpgaloader, "-c", "digilent_hs3", fn])


class LitefuryPlatform(_BasePlatform):
    device      = "xc7a100t"
    package     = "fgg484"


class NitefuryIIPlatform(_BasePlatform):
    device      = "xc7a200t"
    package     = "fbg484"


class TestCase(unittest.TestCase):
    def test_smoke(self):
        from .test.blinky import Blinky
        NitefuryIIPlatform().build(Blinky(), do_build=False)


if __name__ == "__main__":
    from .test.blinky import *
    NitefuryIIPlatform().build(Blinky(), do_program=True)



import os
import subprocess

from amaranth.build import *
from amaranth.vendor import XilinxPlatform
from .resources import *

"""
Example Usage:
    platform = CModA7_35Platform(toolchain="Symbiflow")
    platform.build(Top(), do_program=True)

Supported programmer:
    openocd
"""

__all__ = ["CmodA7_15Platform", "CmodA7_35Platform"]


class _CmodA7Platform(XilinxPlatform):
    package     = "cpg236"
    speed       = "1"
    default_clk = "clk12"
    resources   = [
        Resource("clk12", 0, Pins("L17", dir="i"),
                 Clock(12e6), Attrs(IOSTANDARD="LVCMOS33")),

        *LEDResources(pins="A17 C16", attrs=Attrs(IOSTANDARD="LVCMOS33")),

        RGBLEDResource(0, r="C17", g="B16", b="B17", invert=True,
            attrs=Attrs(IOSTANDARD="LVCMOS33")),

        *ButtonResources(pins="A18 B18", attrs=Attrs(IOSTANDARD="LVCMOS33")),

        UARTResource(0,
            rx="J17", tx="J18",
            attrs=Attrs(IOSTANDARD="LVCMOS33")
        ),

        *SPIFlashResources(0,
            cs_n="K19", clk="E19", copi="D19", cipo="D18", wp_n="G18",
            hold_n="F18",
            attrs=Attrs(IOSTANDARD="LVCMOS33")
        ),

        SRAMResource(0,
            cs_n="N19", oe_n="P19", we_n="R19",
            a="M18 M19 K17 N17 P17 P18 R18 W19 U19 V19 W18 T17 T18 U17 U18 V16 W16 W17 V15",
            d="W15 W13 W14 U15 U16 V13 V14 U14"),

        # One-wire interface to crypto authentication device
        # May not be populated on the board
        Resource("atsha204a", 0, Pins("D17", dir="io"),
            Attrs(IOSTANDARD="LVCMOS33"))
    ]
    connectors  = [
        Connector("pmod", 0, "G17 G19 N18 L18 - - H17 H19 J19 K18 - -"), # JA

        # Pin 24/25 are VCC and GND
        # Pin 15/16 are analog (XADC)
        Connector("gpio", 0,
            """
            M3  L3 A16  K3 C15  H1 A15 B15 A14  J3  J1  K2
            L1  L2   -   -  M1  N3  P3  M2  N1  N2  P1   -
             -  R3  T3  R2  T1  T2  U1  W2  V2  W3  V3  W5
            V4  U4  V5  W4  U5  U2  W6  U3  U7  W7  U8  V8
            """),

        Connector("xadc", 0, {
            "vaux4_n":   "G2",
            "vaux4_p":   "G3",
            "vaux12_n":  "J2",
            "vaux12_p":  "H2"
        })
    ]

    def toolchain_program(self, products, name):
        with products.extract("{}.bit".format(name)) as bitstream_filename:
            subprocess.check_call(["openFPGALoader",
                "-b", "cmoda7_35t",
                "{}".format(bitstream_filename)
            ])


class CmodA7_15Platform(_CmodA7Platform):
    device      = "xc7a15t"

class CmodA7_35Platform(_CmodA7Platform):
    device      = "xc7a35t"

if __name__ == "__main__":
    from .test.blinky import *
    CmodA7_35Platform().build(Blinky(), do_program=True)
